/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.aktuell;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Aufteilungspunkte der aktuellen Konfiguration.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: AufteilungspunkteAktuelleKonfiguration.java 9064 2008-05-21 09:37:11Z gieseler $
 * 
 */

public class AufteilungspunkteAktuelleKonfiguration extends
		ArrayList<AufteilungspunktAktuelleKonfiguration> {

	/**
	 * serialVersionUID.
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Konstruktor. Erzeugt die Liste der Aufteilungspunkte, die aus der
	 * aktuellenKonfiguration zu ber&uuml;cksichtigen sind.
	 * 
	 * @param aktuelleKonfiguration
	 *            aktuelle Konfiguration
	 * @param dav
	 *            DaV-Verbindung
	 */
	public AufteilungspunkteAktuelleKonfiguration(
			final SINAktuelleKonfiguration aktuelleKonfiguration,
			ClientDavConnection dav) {
		for (SystemObject objekt : aktuelleKonfiguration.getAQ()) {
			this.add(new AufteilungspunktAktuelleKonfiguration(objekt, dav));
		}

		for (SystemObject objekt : aktuelleKonfiguration.getFreieMarker()) {
			this.add(new AufteilungspunktAktuelleKonfiguration(objekt, dav));
		}

	}

	/**
	 * Gibt die Liste der Aufteilungspunkte zur&uuml;ck, die NUR durch
	 * Koordinaten bestimmt sind.
	 * 
	 * @return Liste von Aufteilungspunkten
	 */
	public List<AufteilungspunktAktuelleKonfiguration> getAufteilungspunkteKoordinaten() {
		List<AufteilungspunktAktuelleKonfiguration> liste = new ArrayList<AufteilungspunktAktuelleKonfiguration>();
		for (AufteilungspunktAktuelleKonfiguration punkt : this) {
			if (punkt.hatKoordinaten() && !punkt.hatLinienreferenz()) {
				liste.add(punkt);
			}
		}
		return liste;
	}

	/**
	 * Gibt die Liste der Aufteilungspunkte zur&uuml;ck, die durch eine
	 * Linienreferenz bestimmt sind.
	 * 
	 * @return Liste von Aufteilungspunkten
	 */
	public List<AufteilungspunktAktuelleKonfiguration> getAufteilungspunkteLinienreferenz() {
		List<AufteilungspunktAktuelleKonfiguration> liste = new ArrayList<AufteilungspunktAktuelleKonfiguration>();
		for (AufteilungspunktAktuelleKonfiguration punkt : this) {
			if (punkt.hatLinienreferenz()) {
				liste.add(punkt);
			}
		}
		
		return liste;
	}
}
