/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.kb;

import java.util.ArrayList;

import com.vividsolutions.jts.geom.Coordinate;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Punkt;

/**
 * Abstrakte Implementation Konfigurationsobjekt.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: AbstraktKonfigurationsObjekt.java 9192 2008-05-26 12:56:20Z gieseler $
 * 
 */
public abstract class AbstraktKonfigurationsObjekt implements
		KonfigurationsObjekt {

	/**
	 * Koordinatenliste.
	 */
	protected ArrayList<WGS84Punkt> koordinaten = new ArrayList<WGS84Punkt>();

	/**
	 * L&auml;nge.
	 */
	private double laenge = 0;

	/**
	 * PID.
	 */
	protected String pid;

	/**
	 * Name.
	 */
	protected String name;

	/**
	 * zugeh&ouml;riger Typ im Datenkatalog.
	 */
	SystemObjectType davtyp = null;

	/**
	 * Konstruktor mit PID und Name.
	 * 
	 * @param pid
	 *            PID
	 * @param name
	 *            Name
	 */
	public AbstraktKonfigurationsObjekt(final String pid, final String name) {
		this.pid = pid;
		this.name = name;
	}

	/**
	 * Erzeugt eine DaV-Data Instanz.
	 * 
	 * @param dav
	 *            Datenverteiler-Verbindung.
	 * @param atg
	 *            Attributgruppe
	 * @param asp
	 *            Aspekt
	 * @return Data
	 */
	public Data createData(final ClientDavConnection dav, final String atg,
			final String asp) {
		AttributeGroup attributeGroup = dav.getDataModel().getAttributeGroup(
				atg);
		// Aspect aspect = dav.getDataModel().getAspect(asp);
		// DataDescription dataDescription = new DataDescription(attributeGroup,
		// aspect);

		return dav.createData(attributeGroup);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.KonfigurationsObjekt#getDavtype()
	 */
	public SystemObjectType getDavtype() {
		return davtyp;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.KonfigurationsObjekt#getKoordinaten()
	 */
	public ArrayList<WGS84Punkt> getKoordinaten() {
		return this.koordinaten;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.KonfigurationsObjekt#getLaenge()
	 */
	public double getLaenge() {
		return laenge;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.KonfigurationsObjekt#getName()
	 */
	public String getName() {
		return name;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.KonfigurationsObjekt#getPid()
	 */
	public String getPid() {
		return pid;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.KonfigurationsObjekt#setKoordinaten(java.util.ArrayList)
	 */
	public void setKoordinaten(final ArrayList<WGS84Punkt> punkte) {
		this.koordinaten = punkte;
	}

	/**
	 * Setzt die Koordinaten auf der Basis von Feature-Koordinaten.
	 * 
	 * @param featureKoordinatenWGS84
	 *            Feature-Koordinaten
	 */
	public void setKoordinaten(final Coordinate[] featureKoordinatenWGS84) {
		for (int i = 0; i < featureKoordinatenWGS84.length; i++) {
			// WGS84Koordinate w = GeoTransformation
			// .uTMnachWGS84Punkt(new UTMKoordinate(
			// featureKoordinatenWGS84[i].x,
			// featureKoordinatenWGS84[i].y, 32));
			this.koordinaten.add(new WGS84Punkt(featureKoordinatenWGS84[i].x,
					featureKoordinatenWGS84[i].y));
		}
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.KonfigurationsObjekt#setLaenge(double)
	 */
	public void setLaenge(final double laenge) {
		this.laenge = laenge;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.KonfigurationsObjekt#setName(java.lang.String)
	 */
	public void setName(final String name) {
		this.name = name;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.KonfigurationsObjekt#setPid(java.lang.String)
	 */
	public void setPid(final String pid) {
		this.pid = pid;
	}
}
