/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.kb;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.kex.bwsin.konfig.Location;
import de.bsvrz.kex.bwsin.konfig.SegmentFactory;
import de.bsvrz.kex.bwsin.konfig.StrassenAbschnitt;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Punkt;

/**
 * Abstrakte Implementation eines Stra&szligensegmentes.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: AbstraktStrassenSegment.java 10390 2008-07-09 15:06:39Z gieseler $
 */
public abstract class AbstraktStrassenSegment extends
		AbstraktKonfigurationsObjekt implements SINStrassenSegment {

	/**
	 * Die Stra&szlig;enteilsegmente, aus denen das Stra&szlig;ensegment
	 * besteht.
	 */
	// protected ArrayList<KonfigurationsObjekt> teilSegmente = new
	// ArrayList<KonfigurationsObjekt>();
	/**
	 * Die Stra&szlig;enabschnitte, aus denen das Segment gebildet wird.
	 */
	protected List<StrassenAbschnitt> abschnitte = null;

	/**
	 * Die Stra&szlig;e, zu der das Stra&szlig;ensegment geh&ouml;rt.
	 */
	protected KonfigurationsObjekt zuStrasse = null;

	/**
	 * Location von der sich das Stra&szlig;ensegment wegbewegt.
	 */
	protected Location locationVon = null;

	/**
	 * Location auf die sich das Stra&szlig;ensegment zubewegt.
	 */
	protected Location locationNach = null;

	/**
	 * Konstruktor mit PID und Name.
	 * 
	 * @param pid
	 *            PID
	 * @param name
	 *            Name
	 */
	public AbstraktStrassenSegment(String pid, String name) {
		super(pid, name);
	}

	/**
	 * Setzt den Stra&szlig;enabschnitt, aus dem das Segment gebildet wird.
	 * 
	 * @param abschnitt
	 *            Stra&szlig;enabschnitt
	 */
	public void addAbschnitt(StrassenAbschnitt abschnitt) {
		if (this.abschnitte == null) {
			this.abschnitte = new ArrayList<StrassenAbschnitt>();
		}

		this.abschnitte.add(abschnitt);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.SINStrassenSegment#aufteilenOffset(float)
	 */
	public boolean aufteilenOffset(final float offset) {
		float aoffset = offset;
		if (abschnitte != null) {
			for (StrassenAbschnitt abschnitt : abschnitte) {
				if (abschnitt.aufteilenOffset(aoffset)) {
					return true;
				}
				aoffset -= abschnitt.getLaenge();
			}
		}
		return false;
	}

	// /**
	// * {@inheritDoc}
	// *
	// * @see
	// de.bsvrz.kex.bwsin.konfig.kb.SINStrassenSegment#addTeilSegment(de.bsvrz.kex.bwsin.konfig.kb.SINSTS)
	// */
	// public void addTeilSegment(final SINSTS teilsegment) {
	// teilSegmente.add(teilsegment);
	// laenge += teilsegment.getLaenge();
	// }

	// /**
	// * {@inheritDoc}
	// *
	// * @see
	// de.bsvrz.kex.bwsin.konfig.kb.SINStrassenSegment#addTeilSegmente(java.util.List)
	// */
	// public void addTeilSegmente(final List<KonfigurationsObjekt>
	// teilsegmente) {
	// teilSegmente.addAll(teilsegmente);
	// for (KonfigurationsObjekt sts : teilsegmente) {
	// laenge += sts.getLaenge();
	// }
	// }

	/**
	 * Gibt die Stra&szlig;enabschnitte zur&uuml;ck, aus denen das Segment
	 * gebildet wird.
	 * 
	 * @return Liste von Stra&szlig;enabschnitten
	 */
	public List<StrassenAbschnitt> getAbschnitte() {
		return abschnitte;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.AbstraktKonfigurationsObjekt#getKoordinaten()
	 */
	@Override
	public ArrayList<WGS84Punkt> getKoordinaten() {
		ArrayList<WGS84Punkt> koord = new ArrayList<WGS84Punkt>();

		for (KonfigurationsObjekt sts : getTeilSegmente()) {
			koord.addAll(sts.getKoordinaten());
		}

		return new ArrayList<WGS84Punkt>(koord);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.AbstraktKonfigurationsObjekt#getLaenge()
	 */
	@Override
	public double getLaenge() {
		double laenge = 0;
		for (KonfigurationsObjekt k : getTeilSegmente()) {
			laenge += k.getLaenge();
		}

		return laenge;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.SINStrassenSegment#getLocationNach()
	 */
	public Location getLocationNach() {
		return locationNach;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.SINStrassenSegment#getLocationVon()
	 */
	public Location getLocationVon() {
		return locationVon;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.SINStrassenSegment#getTeilSegmente()
	 */
	public List<KonfigurationsObjekt> getTeilSegmente() {
		List<KonfigurationsObjekt> teilsegmente = new ArrayList<KonfigurationsObjekt>();
		if (abschnitte != null) {
			for (StrassenAbschnitt abschnitt : abschnitte) {
				teilsegmente.addAll(abschnitt.getSTS());
			}
		}

		return teilsegmente;
	}

	/**
	 * Korrogiert die PID und den Teil des Namens, der durch die PID gebildet
	 * wird.
	 * 
	 * @param korrigiertePid
	 *            neue PID
	 */
	public void korrigierePidUndNamen(String korrigiertePid) {
		pid = korrigiertePid;
		String namezusatz = (name.indexOf(" ") != -1 && name.substring(
				name.indexOf(" ")).length() > 1) ? name.substring(name
				.indexOf(" ") + 1) : "";

		name = SegmentFactory.bildeSegmentNamen(pid, namezusatz);
	}

	/**
	 * Setzt den Stra&szlig;enabschnitt, aus dem das Segment gebildet wird.
	 * 
	 * @param abschnitte
	 *            Stra&szlig;enabschnitt
	 */
	public void setAbschnitte(List<StrassenAbschnitt> abschnitte) {
		this.abschnitte = abschnitte;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.SINStrassenSegment#setLocationNach(de.bsvrz.kex.bwsin.konfig.Location)
	 */
	public void setLocationNach(final Location nach) {
		locationNach = nach;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.SINStrassenSegment#setLocationVon(de.bsvrz.kex.bwsin.konfig.Location)
	 */
	public void setLocationVon(final Location von) {
		locationVon = von;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.SINStrassenSegment#setZuStrasse(de.bsvrz.kex.bwsin.konfig.kb.KonfigurationsObjekt)
	 */
	public void setZuStrasse(final KonfigurationsObjekt strasse) {
		this.zuStrasse = strasse;
	}
}
