/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.okstra;

/**
 * Exception fr Ausnahmen bei der Konvertierung von Feature-Attributen.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: IllegalAttributeConversionException.java 10401 2008-07-10 09:06:04Z gieseler $
 * 
 */
public class IllegalAttributeConversionException extends Exception {

	/**
	 * serialVersionUID.
	 */
	private static final long serialVersionUID = 8731532752475259683L;

	/**
	 * Constructs an error message based on expected and invalid.
	 * 
	 * @param expected
	 *            the expected AttributeType.
	 * @param invalid
	 *            the attribute that does not validate against expected.
	 * 
	 * @return an error message reporting the problem.
	 */
	static String errorMessage(Class<?> expected, Object invalid) {
		String message = "expected " //$NON-NLS-1$
				+ ((expected == null) ? "null" : expected.getName()); //$NON-NLS-1$
		message += (" , but got " //$NON-NLS-1$
		+ ((invalid == null) ? "null" : invalid.getClass().getName())); //$NON-NLS-1$

		return message;
	}

	/** The expected type. */
	@SuppressWarnings("unchecked")
	private final Class expected;

	/** The object that does not match the expected type. */
	private final Object invalid;

	/**
	 * Constructor that makes the message given the expected and invalid.
	 * 
	 * @param expected
	 *            the expected AttributeType.
	 * @param invalid
	 *            the attribute that does not validate against expected.
	 */
	public IllegalAttributeConversionException(Class<?> expected, Object invalid) {
		this(expected, invalid, null);
	}

	/**
	 * Constructor that makes the message given the expected and invalid, along
	 * with the root cause.
	 * 
	 * @param expected
	 *            the expected AttributeType.
	 * @param invalid
	 *            the attribute that does not validate against expected.
	 * @param cause
	 *            the root cause of the error.
	 */
	public IllegalAttributeConversionException(Class<?> expected,
			Object invalid, Throwable cause) {
		super(errorMessage(expected, invalid), cause);
		this.expected = expected;
		this.invalid = invalid;
	}

	/**
	 * Constructor with message argument.
	 * 
	 * @param message
	 *            Reason for the exception being thrown
	 */
	public IllegalAttributeConversionException(String message) {
		super(message);
		expected = null;
		invalid = null;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see java.lang.Throwable#toString()
	 */
	@Override
	public String toString() {
		String message = "IllegalAttributeConversion: " //$NON-NLS-1$
				+ ((expected == null) ? "null" : expected.getName()); //$NON-NLS-1$

		message += (" , but got " //$NON-NLS-1$
		+ ((invalid == null) ? "null" : invalid.getClass().getName())); //$NON-NLS-1$

		return message;
	}
}
