package de.bsvrz.kex.bwsin.okstra;

// -----------------------------------------------------------------------------
// SAXExample.java
// -----------------------------------------------------------------------------

/*
 * =============================================================================
 * Copyright (c) 1998-2007 Jeffrey M. Hunter. All rights reserved.
 * 
 * All source code and material located at the Internet address of
 * http://www.idevelopment.info is the copyright of Jeffrey M. Hunter and
 * is protected under copyright laws of the United States. This source code may
 * not be hosted on any other site without my express, prior, written
 * permission. Application to host any of the material elsewhere can be made by
 * contacting me at jhunter@idevelopment.info.
 *
 * I have made every effort and taken great care in making sure that the source
 * code and other content included on my web site is technically accurate, but I
 * disclaim any and all responsibility for any loss, damage or destruction of
 * data or any other property which may arise from relying on it. I will in no
 * case be liable for any monetary damages arising from such loss, damage or
 * destruction.
 * 
 * As with any code, ensure to test this code in a development environment 
 * before attempting to run it in production.
 * =============================================================================
 */

import java.io.File;
import java.io.FileReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Stack;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.geotools.xml.handlers.BWSIN_ElementHandlerFactory;
import org.geotools.xml.handlers.DocumentHandler;
import org.geotools.xml.handlers.ElementHandlerFactory;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import de.bsvrz.kex.bwsin.konfig.okstra.parser.SINElementHandler;
import de.bsvrz.kex.bwsin.konfig.okstra.parser.SINFeatureHandler;
import de.bsvrz.kex.bwsin.konfig.okstra.parser.SINXMLHandler;

/**
 * -----------------------------------------------------------------------------
 * Demonstrate how to use SAX.
 * 
 * @version 1.0
 * @author Jeffrey M. Hunter (jhunter@idevelopment.info)
 * @author http://www.idevelopment.info
 *         -----------------------------------------------------------------------------
 */

public class SINParserTest {

	// Store the locator
	// Locator locator;

	protected final static Logger logger = getLogger();

	/*
	 * * +---------------------------------------------+ * | METHOD: fileToURL | *
	 * +---------------------------------------------+
	 */
	static URL fileToURL(File file) {

		String path = file.getAbsolutePath();
		String fSep = System.getProperty("file.separator");
		if (fSep != null && fSep.length() == 1) {
			path = path.replace(fSep.charAt(0), '/');
		}
		if (path.length() > 0 && path.charAt(0) != '/') {
			path = '/' + path;
		}

		try {
			return new URL("file", null, path);
		} catch (java.net.MalformedURLException e) {
			throw new Error("unexpected MalformedException");
		}

	}

	private static final Logger getLogger() {
		Logger l = Logger.getLogger("BWSINWFS");
		l.setLevel(Level.ALL);
		for (Handler h : l.getHandlers()) {
			h.setLevel(Level.ALL);
		}
		return l;
	}

	/*
	 * * +---------------------------------------------+ * | METHOD: main | *
	 * +---------------------------------------------+
	 */
	static public void main(String[] argv) {
		SINParserTest mp = new SINParserTest();
		mp.parse();
	}

	SINFeatureHandler fh = null;
	private int level;

	// the stack of handlers
	private final Stack handlers = new Stack();
	// the base handler for the document
	private final DocumentHandler document = null;

	private final ElementHandlerFactory ehf = new BWSIN_ElementHandlerFactory(
			logger);

	List<SINFeatureHandler> fliste = new ArrayList<SINFeatureHandler>();

	private SINElementHandler current;

	public SINParserTest() {
		super();
	}

	public void parse() {

		try {

			// if (argv.length != 1) {
			// // must pass in the name of the XML file
			// System.err.println("Usage: java SAXExample filename");
			// System.exit(1);
			// }

			// String filename =
			// "/home/gysi/workspace/wfsclient/okstrads/features_okstra_Strecke_kurz.xml";
			// String filename = "C:\\Dokumente und
			// Einstellungen\\Steffen\\Eigene
			// Dateien\\workspace\\wfsclient\\okstrads\\features_okstra_Strecke_kurz.xml";
			String filename = "C:\\Dokumente und Einstellungen\\Steffen\\Eigene Dateien\\workspace\\wfsclient\\okstrads\\features_okstra_Strecke.xml";

			XMLReader xr = XMLReaderFactory.createXMLReader();
			SINXMLHandler handler = new SINXMLHandler();
			xr.setContentHandler(handler);
			xr.setErrorHandler(handler);

			FileReader r = new FileReader(filename);
			xr.parse(new InputSource(r));

			System.out.println("Ergebnis: " + handler.getFeatures().size()
					+ " Feature(s)");

			for (SINFeatureHandler fh : handler.getFeatures().values()) {
				System.out.println("Feature: " + fh.getName());
				for (Object o : fh.getAttributeWerte("endet_bei_Station")) {
					System.out.println("endet_bei_Station: " + o);
				}
			}

		} catch (Exception e) {
			System.out.println(e.toString());
		}

	}

}
