/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.vew;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.jdesktop.swingworker.SwingWorker;

import de.bsvrz.kex.bwsin.konfig.ImportException;

/**
 * Statischer Import Netz mit grafischem Benutzerinterface.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: BWSINgui.java 10429 2008-07-10 12:09:58Z gieseler $
 * 
 */
public class BWSINgui extends BWSIN implements ActionListener, DocumentListener {

	/**
	 * Hauptklasse SWE2.BW-SIN mit GUI.
	 * 
	 * @author BitCtrl Systems GmbH, Gieseler
	 * @version $Id: BWSINgui.java 10429 2008-07-10 12:09:58Z gieseler $
	 * 
	 */
	class SINWorker extends SwingWorker<Integer, Object> {
		/**
		 * {@inheritDoc}
		 * 
		 * @see org.jdesktop.swingworker.SwingWorker#doInBackground()
		 */
		@Override
		public Integer doInBackground() throws Exception {
			try {
				importieren();
			} catch (Exception e) {
				System.err.println(e.getMessage());
			}

			return 0;
		}

		/**
		 * {@inheritDoc}
		 * 
		 * @see org.jdesktop.swingworker.SwingWorker#done()
		 */
		@Override
		protected void done() {
			logFenster.enableSchliessen();
		}

	}

	/**
	 * serialVersionUID.
	 */
	private static final long serialVersionUID = 2113316346767696073L;

	/**
	 * F&uuml;gt eine Komponente hinzu.
	 * 
	 * @param cont
	 *            Container
	 * @param gbl
	 *            Gridbaglayout
	 * @param c
	 *            Komponente
	 * @param x
	 *            X-Koordinate
	 * @param y
	 *            Y-Koordinate
	 * @param width
	 *            Breie
	 * @param height
	 *            H&ouml;he
	 * @param weightx
	 *            horizontale Verteilung
	 * @param weighty
	 *            vertikale Verteilung
	 * @param fill
	 *            F&uuml;llung
	 */
	private static void addComponent(Container cont, GridBagLayout gbl,
			Component c, int x, int y, int width, int height, double weightx,
			double weighty, int fill) {
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.insets = new Insets(5, 5, 5, 5);
		gbc.fill = fill;
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = width;
		gbc.gridheight = height;
		gbc.weightx = weightx;
		gbc.weighty = weighty;
		gbl.setConstraints(c, gbc);
		cont.add(c);
	}

	/**
	 * Worker-Thread.
	 */
	private SINWorker workerThread = null;

	/**
	 * Layout.
	 */
	private GridBagLayout gridbaglayout;
	// private int _SCREEN_WIDTH;
	// private int _SCREEN_HEIGHT;

	/**
	 * Start-Button Import.
	 */
	private JButton importButton;

	// private JPanel _panel;

	/**
	 * Kopfzeile.
	 */
	private JLabel kopfZeile = null;

	/**
	 * Textfeld 'Pfad Konfigurationsbereich'.
	 */
	private JTextField kbPfadTextFeld;

	/**
	 * Auswahlbutton 'Pfad Konfigurationsbereich'.
	 */
	private JButton kbPfadButton;

	/**
	 * Textfeld 'Okstra-Parameter'.
	 */
	private JTextField okstraParamTextFeld;

	/**
	 * Auswahlbutton Location-Code-Liste.
	 */
	private JButton okstraParamButton;

	/**
	 * Textfeld 'Konfigurationsverantwortlicher'.
	 */
	private JTextField kvTextFeld;

	/**
	 * Textfeld 'Konfigurationsbereich PID'.
	 */
	private JTextField kbPidTextFeld;

	/**
	 * Textfeld 'Name Konfigurationsbereich'.
	 */
	private JTextField kbNameTextFeld;

	/**
	 * Textfeld 'Info Konfigurationsbereich'.
	 */
	private JTextArea kbInfoTextFeld;

	/**
	 * Textfeld 'Location-Code-Liste'.
	 */
	private JTextField lclTextFeld;

	/**
	 * Auswahlbutton Location-Code-Liste.
	 */
	private JButton lclButton;

	/**
	 * Textfeld 'Abweichungsprotokoll'.
	 */
	private JTextField protokollTextFeld;

	/**
	 * Auswahlbutton Abweichungsprotokoll.
	 */
	private JButton protokollButton;

	/**
	 * Textfeld 'Netzreferenzen'.
	 */
	private JTextField referenzenTextFeld;

	/**
	 * Auswahlbutton Netzreferenzen.
	 */
	private JButton referenzenButton;

	/**
	 * Auswahlbox 'mit Abweichungen'.
	 */
	private JComboBox mitAbweichungenBox;

	/**
	 * Frame.
	 */
	JFrame frame;

	/**
	 * Das Fenster f&uuml;r die Log-Ausgaben w&auml;hrend des Importes.
	 */
	private LogFenster logFenster;

	/**
	 * Debug-Logger.
	 */
	// private Debug debug;
	/**
	 * Konstruktor.
	 */
	public BWSINgui() {
		super();
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {

		String property = ""; //$NON-NLS-1$

		if (e.getSource() == this.importButton) {
			try {
				SINProperties.getInstance().setProperty(
						SINProperties.PROP_KEY_LCL, lclTextFeld.getText());
				SINProperties.getInstance()
						.setProperty(SINProperties.PROP_KEY_KBINFO,
								kbInfoTextFeld.getText());
				SINProperties.getInstance().setProperty(
						SINProperties.PROP_KEY_QUELLDATENBASIS,
						okstraParamTextFeld.getText());
				SINProperties.getInstance().setProperty(
						SINProperties.PROP_KEY_KV, kvTextFeld.getText());
				SINProperties.getInstance().setProperty(
						SINProperties.PROP_KEY_KBPID, kbPidTextFeld.getText());
				SINProperties.getInstance()
						.setProperty(SINProperties.PROP_KEY_KBNAME,
								kbNameTextFeld.getText());
				SINProperties.getInstance()
						.setProperty(SINProperties.PROP_KEY_KBPFAD,
								kbPfadTextFeld.getText());
				SINProperties.getInstance().setProperty(
						SINProperties.PROP_KEY_REFERENZEN,
						referenzenTextFeld.getText());
				SINProperties.getInstance().setProperty(
						SINProperties.PROP_KEY_ABW_PROTOKOLLDATEI,
						protokollTextFeld.getText());

				checkParameter();
				createLog();

				workerThread = new SINWorker();
				workerThread.execute();

				this.frame.setCursor(Cursor
						.getPredefinedCursor(Cursor.WAIT_CURSOR));
				enableComponent(this.frame, false);
			} catch (ImportException ie) {
				JOptionPane.showMessageDialog(this.frame, ie.getMessage(),
						"Fehler", JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
			} catch (Exception ex) {
				ex.printStackTrace();
				JOptionPane.showMessageDialog(this.frame, ex.getMessage(),
						"Fehler", JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
			}
			return;
		}

		// Texteingaben
		if (e.getSource() == this.lclTextFeld) {
			SINProperties.getInstance().setProperty(SINProperties.PROP_KEY_LCL,
					lclTextFeld.getText());
			return;
		} else if (e.getSource() == this.okstraParamTextFeld) {
			SINProperties.getInstance().setProperty(
					SINProperties.PROP_KEY_QUELLDATENBASIS,
					okstraParamTextFeld.getText());
			return;
		} else if (e.getSource() == this.kvTextFeld) {
			SINProperties.getInstance().setProperty(SINProperties.PROP_KEY_KV,
					kvTextFeld.getText());
			return;
		} else if (e.getSource() == this.kbPidTextFeld) {
			SINProperties.getInstance().setProperty(
					SINProperties.PROP_KEY_KBPID, kbPidTextFeld.getText());
			return;
		} else if (e.getSource() == this.kbNameTextFeld) {
			SINProperties.getInstance().setProperty(
					SINProperties.PROP_KEY_KBNAME, kbNameTextFeld.getText());
			return;
		} else if (e.getSource() == this.kbInfoTextFeld) {
			SINProperties.getInstance().setProperty(
					SINProperties.PROP_KEY_KBINFO, kbInfoTextFeld.getText());
			return;
		} else if (e.getSource() == this.mitAbweichungenBox) {
			SINProperties.getInstance().setProperty(SINProperties.PROP_KEY_ABW,
					(String) mitAbweichungenBox.getSelectedItem());
			return;
		}

		// Buttons
		int mode = JFileChooser.FILES_ONLY;
		JTextField zielfeld = null;

		// Handle open button action.
		if (e.getSource() == this.okstraParamButton) {
			property = SINProperties.PROP_KEY_QUELLDATENBASIS;
			zielfeld = this.okstraParamTextFeld;
		} else if (e.getSource() == lclButton) {
			property = SINProperties.PROP_KEY_LCL;
			zielfeld = this.lclTextFeld;
		} else if (e.getSource() == kbPfadButton) {
			mode = JFileChooser.DIRECTORIES_ONLY;
			property = SINProperties.PROP_KEY_KBPFAD;
			zielfeld = this.kbPfadTextFeld;
		} else if (e.getSource() == protokollButton) {
			property = SINProperties.PROP_KEY_ABW_PROTOKOLLDATEI;
			zielfeld = this.protokollTextFeld;
		} else if (e.getSource() == referenzenButton) {
			property = SINProperties.PROP_KEY_REFERENZEN;
			zielfeld = this.referenzenTextFeld;
		} else {
			return;
		}

		String filename = SINProperties.getInstance().getProperty(property);
		if (filename == null) {
			filename = "";
		}

		JFileChooser fc = new JFileChooser();

		fc.setSelectedFile(new File(filename));
		fc.setFileSelectionMode(mode);

		int returnVal = fc.showOpenDialog(BWSINgui.this.frame);

		if (returnVal == JFileChooser.APPROVE_OPTION) {
			String file = fc.getSelectedFile().toString();
			SINProperties.getInstance().setProperty(property, file);
			if (zielfeld != null) {
				zielfeld.setText(file);
			}
		}

	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see javax.swing.event.DocumentListener#changedUpdate(javax.swing.event.DocumentEvent)
	 */
	public void changedUpdate(DocumentEvent arg0) {
		/**
		 * 
		 */
	}

	/**
	 * Erzeugt das Ausgabefenster f&uuml;r die LOG-Ausgaben.
	 */
	private void createLog() {
		logFenster = new LogFenster(this);
	}

	// private void createWaitDialog() {
	//
	// int breite = 300;
	// int hoehe = 20;
	// waitdialog = new JDialog(this._frame, "Der Import wird ausgefhrt",
	// true);
	// progressbar = new JProgressBar();
	// progressbar.setIndeterminate(true);
	// waitdialog.add(progressbar);
	// waitdialog.setSize(new Dimension(breite, hoehe));
	// waitdialog.setLocation(this._frame.getLocation().x
	// + ((this._frame.getSize().width - breite) / 2), this._frame
	// .getLocation().y
	// + ((this._frame.getSize().height - hoehe) / 2));
	// waitdialog.setAlwaysOnTop(true);
	// waitdialog.setResizable(false);
	// // waitdialog.setUndecorated(true);
	// waitdialog.setVisible(true);
	// }

	/**
	 * Gibt eine Komponente frei.
	 * 
	 * @param cont
	 *            Komponente
	 * @param enable
	 *            Freigabe ja/nein
	 */
	void enableComponent(final Container cont, final boolean enable) {
		for (Component c : cont.getComponents()) {
			c.setEnabled(enable);
			if (c instanceof Container) {
				enableComponent((Container) c, enable);
			}
		}
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see javax.swing.event.DocumentListener#insertUpdate(javax.swing.event.DocumentEvent)
	 */
	public void insertUpdate(DocumentEvent arg0) {
		/**
		 * 
		 */
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see javax.swing.event.DocumentListener#removeUpdate(javax.swing.event.DocumentEvent)
	 */
	public void removeUpdate(DocumentEvent arg0) {
		/**
		 * 
		 */
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.vew.BWSIN#startImport()
	 */
	@Override
	public void startImport() {
		gridbaglayout = new GridBagLayout();
		int screenWidth = Toolkit.getDefaultToolkit().getScreenSize().width;
		int screenHeight = Toolkit.getDefaultToolkit().getScreenSize().height;

		this.frame = new JFrame("Statischer Import Netzdaten"); //$NON-NLS-1$
		this.frame.setLayout(this.gridbaglayout);
		this.frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.frame.setSize(new Dimension(900, 550));
		this.frame.setLocation((screenWidth - this.frame.getSize().width) / 2,
				(screenHeight - this.frame.getSize().height) / 2);

		this.frame.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				if (e.getComponent() == frame) {
					super.windowClosing(e);
					dav.disconnect(false, ""); //$NON-NLS-1$
				}
			}
		});

		this.kopfZeile = new JLabel("Importparameter"); //$NON-NLS-1$
		this.kopfZeile.setFont(new Font(null, 0, 30));

		this.okstraParamTextFeld = new JTextField(SINProperties.getInstance()
				.getProperty(SINProperties.PROP_KEY_QUELLDATENBASIS));
		this.okstraParamTextFeld.addActionListener(this);
		this.okstraParamButton = new JButton("Auswhlen"); //$NON-NLS-1$
		this.okstraParamButton.addActionListener(this);

		this.lclTextFeld = new JTextField(SINProperties.getInstance()
				.getProperty(SINProperties.PROP_KEY_LCL));
		this.lclTextFeld.addActionListener(this);
		this.lclButton = new JButton("Auswhlen"); //$NON-NLS-1$
		this.lclButton.addActionListener(this);

		this.kbPfadTextFeld = new JTextField(SINProperties.getInstance()
				.getProperty(SINProperties.PROP_KEY_KBPFAD));
		this.kbPfadTextFeld.addActionListener(this);
		this.kbPfadButton = new JButton("Auswhlen"); //$NON-NLS-1$
		this.kbPfadButton.addActionListener(this);

		this.kvTextFeld = new JTextField(SINProperties.getInstance()
				.getProperty(SINProperties.PROP_KEY_KV));
		this.kvTextFeld.addActionListener(this);

		this.kbPidTextFeld = new JTextField(SINProperties.getInstance()
				.getProperty(SINProperties.PROP_KEY_KBPID));
		this.kbPidTextFeld.addActionListener(this);
		this.kbNameTextFeld = new JTextField(SINProperties.getInstance()
				.getProperty(SINProperties.PROP_KEY_KBNAME));
		this.kbNameTextFeld.addActionListener(this);
		this.kbInfoTextFeld = new JTextArea(SINProperties.getInstance()
				.getProperty(SINProperties.PROP_KEY_KBINFO));
		this.kbInfoTextFeld.getDocument().addDocumentListener(this);
		this.kbInfoTextFeld.setLineWrap(true);
		this.kbInfoTextFeld.setWrapStyleWord(true);
		// this._kb_info_text.setFont(new Font(null, 0, 18 ));
		this.kbInfoTextFeld.setBorder(new LineBorder(Color.GRAY));

		// t.setFont( new Font(Font.SANS_SERIF, style, size) );
		// this._kb_info_p = new JScrollPane(this._kb_info_text );

		String[] konvOptions = { "Ja", "Nein" }; //$NON-NLS-1$//$NON-NLS-2$
		this.mitAbweichungenBox = new JComboBox(konvOptions);
		if (SINProperties.getInstance().getProperty(SINProperties.PROP_KEY_ABW)
				.equalsIgnoreCase("Ja")) {
			this.mitAbweichungenBox.setSelectedIndex(0);
		} else {
			this.mitAbweichungenBox.setSelectedIndex(1);
		}

		this.mitAbweichungenBox.setBackground(Color.white);
		this.mitAbweichungenBox.setFont(this.mitAbweichungenBox.getFont()
				.deriveFont(Font.PLAIN));
		// this._abweichungen_jn.setFont(new Font(null, Font.PLAIN, 18 ));
		this.mitAbweichungenBox.addActionListener(this);

		this.protokollTextFeld = new JTextField(SINProperties.getInstance()
				.getProperty(SINProperties.PROP_KEY_ABW_PROTOKOLLDATEI));
		this.protokollTextFeld.addActionListener(this);
		this.protokollButton = new JButton("Auswhlen"); //$NON-NLS-1$
		this.protokollButton.addActionListener(this);

		this.referenzenTextFeld = new JTextField(SINProperties.getInstance()
				.getProperty(SINProperties.PROP_KEY_REFERENZEN));
		this.referenzenTextFeld.addActionListener(this);
		this.referenzenButton = new JButton("Auswhlen"); //$NON-NLS-1$
		this.referenzenButton.addActionListener(this);

		this.importButton = new JButton("Import starten"); //$NON-NLS-1$
		this.importButton.setEnabled(true);
		this.importButton.addActionListener(this);

		// this._kb_appr = new JTextField("0,000010");

		JPanel panel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		panel.add(this.kopfZeile);

		addComponent(this.frame, this.gridbaglayout, this.kopfZeile, 0, 0, 3,
				1, 0, 0.1, GridBagConstraints.NONE);

		addComponent(this.frame, this.gridbaglayout, new JLabel(
				"Parameter der Quelldatenbasis:"), //$NON-NLS-1$
				0, 1, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		addComponent(this.frame, this.gridbaglayout, this.okstraParamTextFeld,
				1, 1, 1, 1, 2, 0, GridBagConstraints.HORIZONTAL);
		addComponent(this.frame, this.gridbaglayout, this.okstraParamButton, 2,
				1, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);

		addComponent(this.frame, this.gridbaglayout, new JLabel(
				"Datei LocationCodeListe (LCL):"), //$NON-NLS-1$
				0, 2, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		addComponent(this.frame, this.gridbaglayout, this.lclTextFeld, 1, 2, 1,
				1, 2, 0, GridBagConstraints.HORIZONTAL);
		addComponent(this.frame, this.gridbaglayout, this.lclButton, 2, 2, 1,
				1, 0, 0, GridBagConstraints.HORIZONTAL);

		addComponent(this.frame, this.gridbaglayout, new JLabel(
				"Pfad Konfigurationsbereich:"), //$NON-NLS-1$
				0, 3, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		addComponent(this.frame, this.gridbaglayout, this.kbPfadTextFeld, 1, 3,
				1, 1, 2, 0, GridBagConstraints.HORIZONTAL);
		addComponent(this.frame, this.gridbaglayout, this.kbPfadButton, 2, 3,
				1, 1, 0, 0, GridBagConstraints.HORIZONTAL);

		addComponent(this.frame, this.gridbaglayout, new JLabel(
				"Konfigurationsverantwortlicher:"), //$NON-NLS-1$
				0, 4, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		addComponent(this.frame, this.gridbaglayout, this.kvTextFeld, 1, 4, 2,
				1, 0, 0, GridBagConstraints.HORIZONTAL);

		addComponent(this.frame, this.gridbaglayout, new JLabel(
				"Konfigurationsbereich PID:"), //$NON-NLS-1$
				0, 5, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		addComponent(this.frame, this.gridbaglayout, this.kbPidTextFeld, 1, 5,
				2, 1, 0, 0, GridBagConstraints.HORIZONTAL);

		addComponent(this.frame, this.gridbaglayout, new JLabel(
				"Konfigurationsbereich Name:"), //$NON-NLS-1$
				0, 6, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		addComponent(this.frame, this.gridbaglayout, this.kbNameTextFeld, 1, 6,
				2, 1, 0, 0, GridBagConstraints.HORIZONTAL);

		addComponent(this.frame, this.gridbaglayout, new JLabel(
				"Konfigurationsbereich Info:"), //$NON-NLS-1$
				0, 7, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		addComponent(this.frame, this.gridbaglayout, this.kbInfoTextFeld, 1, 7,
				2, 3, 0, 0, GridBagConstraints.HORIZONTAL);

		addComponent(this.frame, this.gridbaglayout, new JLabel(
				"Abweichungsprotokoll erzeugen:"), //$NON-NLS-1$
				0, 10, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		addComponent(this.frame, this.gridbaglayout, mitAbweichungenBox, 1, 10,
				1, 1, 0, 0, GridBagConstraints.HORIZONTAL);

		addComponent(this.frame, this.gridbaglayout, new JLabel(
				"Referenzendefinition:"), //$NON-NLS-1$
				0, 11, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		addComponent(this.frame, this.gridbaglayout, this.referenzenTextFeld,
				1, 11, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		addComponent(this.frame, this.gridbaglayout, this.referenzenButton, 2,
				11, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);

		addComponent(this.frame, this.gridbaglayout, new JLabel(
				"Protokolldatei:"), //$NON-NLS-1$
				0, 12, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		addComponent(this.frame, this.gridbaglayout, this.protokollTextFeld, 1,
				12, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		addComponent(this.frame, this.gridbaglayout, this.protokollButton, 2,
				12, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);

		// addComponent(this._frame, this._gridbaglayout, new
		// JLabel("Approximation:"),
		// 0, 12, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		// addComponent(this._frame, this._gridbaglayout, this._kb_appr, 1, 12,
		// 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		// addComponent(this._frame, this._gridbaglayout, new JLabel("Grad"),
		// 2, 12, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
		//				
		addComponent(this.frame, this.gridbaglayout, this.importButton, 0, 13,
				3, 0, 1, 1.0, GridBagConstraints.NONE);
		this.frame.setVisible(true);
	}

	/**
	 * Beendet den Worker.
	 */
	public void stopWorker() {
		enableComponent(this.frame, true);
		this.frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		workerThread.cancel(true);
		// boolean st = workerThread.isDone();
	}
}
